require('./main.css')
const html = require('html-loader!./index.html')

class FileDragReader {
	constructor(elementId, options) {
		const e = document.getElementById(elementId)
		e.innerHTML = html
		const drop = document.getElementById('filedrag')
		const fileInput = document.getElementById('fileselect')

		drop.addEventListener('dragover', this.handleDragOver.bind(this), false)
		drop.addEventListener('dragleave', this.handleDragOver.bind(this), false)
		drop.addEventListener('drop', this.handleDrop.bind(this), false)
		fileInput.addEventListener('change', this.handleSelect.bind(this), false)
		this.listeners = {}
		this.options = {}
		const opt = options || { readAs: 'DataURL' }
		this.options.readAs = opt.readAs
	}

	handleDragOver(event) {
		event.stopPropagation()
		event.preventDefault()
		event.dataTransfer.dropEffect = 'copy'
		if (event.type === 'dragover') {
			event.target.classList.add('hover')
		} else if (event.type === 'dragleave') {
			event.target.classList.remove('hover')
		}
	}

	handleDrop(event) {
		event.stopPropagation()
		event.preventDefault()
		event.target.classList.remove('hover')
		this.handleFiles(event.dataTransfer.files)
	}

	handleSelect(event) {
		this.handleFiles(event.target.files)
	}

	async handleFiles(files) {
		if (!files || !files.length) {
			return
		}
		if (this.listeners.start) {
			this.listeners.start()
		}
		const promises = Array.prototype.map.bind(files, e => this.handleFile(e))()
		const results = await Promise.all(promises)
		if (this.listeners.files) {
			this.listeners.files(results)
		}
		if (this.listeners.end) {
			this.listeners.end()
		}
	}
	handleFile(file) {
		return new Promise((resolve) => {
			const fr = new FileReader()
			fr.addEventListener('load', (e) => {
				const rs = { file, data: e.target.result }
				if (this.listeners.file) {
					this.listeners.file(rs)
				}
				resolve(rs)
			})
			fr[`readAs${this.options.readAs}`](file)
		})
	}
	onRead(event, callback) {
		if (['file', 'files', 'start', 'end'].indexOf(event) === -1) {
			throw new Error(`onRead ${event} not support`)
		}
		this.listeners[event] = callback
	}
}

module.exports = FileDragReader

// window.onload = () => {
// 	const reader = new FileDragReader('brickyard-app', { readAs: 'BinaryString' })
// 	reader.onRead('start', () => console.log('start'))
// 	reader.onRead('file', console.log)
// 	reader.onRead('end', () => console.log('end'))
// }
